﻿// download brainboxes library from nuget
using Brainboxes.IO;

// ip address of device to connect to 
string ipAddress = "192.168.2.107";
// create a Brainboxes ED device, can be any of the available classes e.g. ED588, ED527, ED008
var device = new ED588(new TCPConnection(ipAddress));

// setting the IOLine cache timeout will change the background polling interval
// device.IOLineCacheTimeout = 50;

// register for changed to any of the IO lines
device.IOLineChanged += Device_IOLineChanged;

// alternatively can monitor events of just 1 individual line
// device.IOLines[0].IOLineChanged += Device_IOLineChanged;

// or can monitor events of a collection of lines e.g. for the first 4 digital inputs
// device.Inputs.Take(4).AsIOList().IOLineChange += Device_IOLineChanged;

// open a connection to the device to start listening for changes
device.Connect();
Console.WriteLine($"Connected to {device} waiting for changes on IO lines");
// pause main console thread here, press enter to exit application
Console.ReadLine();


/// <summary>
/// any time an IO line of the brainboxes device changes state this function is called
/// there are 3 types of IOLine changes which may invoke this function call:
/// - IOChangeTypes.RisingEdge  - when an IOLine transitions from low to high (0 -> 1)
/// - IOChangeTypes.FallingEdge - when an IOLine transitions from high to low (1-> 0)
/// - IOChangeTypes.Latched - when an IOLine rapidly transitions at least twice
///     . e.g. from high to low to high or vice versa (1 -> 0 -> 1, or 0 -> 1 => 0)
///     Latched only applies to Digital Inputs
/// </summary>
void Device_IOLineChanged(IOLine line, EDDevice device, IOChangeTypes changeType)
{
    Console.WriteLine($"{DateTime.Now.ToString("hh.mm.ss.ffffff")}: Device_IOLineChanged line:{line}, device:{device}, changeType:{changeType}");

    // latched events contain both rising and falling edge events
    // they have simple happened more quickly than the polling interval
    // we report them as latched, but the binary value of the enum is the same as 
    // IOChangeTypes.Latched == IOChangeTypes.RisingEdge & IOChangeTypes.FallingEdge & 0x04
    // 0x07 = 0x1 & 0x02 & 0x4
    // which means all types of rising edges can be detected like this
    if ((changeType & IOChangeTypes.RisingEdge) == IOChangeTypes.RisingEdge)
    {
        Console.WriteLine($"Rising Edge (including latched)");
    }
    // or all types of falling edges can be detected like this
    if ((changeType & IOChangeTypes.FallingEdge) == IOChangeTypes.FallingEdge)
    {
        Console.WriteLine($"Falling Edge (including latched)");
    }

}

/** alternative method of registering only for rising edges (including latched events) **/

// register only for the rising edge events including latched events
// all IOline rising edges across device
device.IOLineRisingEdge += Device_IOLineRisingEdge;
// or can monitor rising events of just 1 individual line
device.IOLines[0].IOLineRisingEdge += Device_IOLineRisingEdge;
// or can monitor events of a collection of lines e.g. digital outputs 2,3,4,5
device.Outputs.Skip(2).Take(4).AsIOList().IOLineRisingEdge += Device_IOLineRisingEdge;

// this will only trigger for IOChangeTypes.RisingEdge and IOChangeTypes.Latched events
void Device_IOLineRisingEdge(IOLine line, EDDevice device, IOChangeTypes changeType)
{
    Console.WriteLine($"{DateTime.Now.ToString("hh.mm.ss.ffffff")}: Device_IOLineRisingEdge line:{line}, device:{device}, changeType:{changeType}");
}